/** JavaScript 'number' primitive type is 64-bit long based on the IEEE 754 standard */
// See also: http://speakingjs.com/es5/ch11.html#number_representation
console.log(Number.MAX_VALUE);
const one: number = Number.parseInt("00000001", 2); // ES2015
if (Number.isNaN(one)) console.log('Number.isNaN(one)');
else console.log('one: ' + one); // '1'
const two: Number = new Number(1 << 1);
console.log('two(2): ' + two.toString(2)); // '10'
const zero: Number = new Number(1 >> 1);
console.log('zero: ' + zero); // '0'

/** JavaScript bitwise operators apply on the first significant 32 bits only! */
// How negative numbers are represented:
const minus_nine: Number = new Number((~9) | 1); // '00000000000000000000000000001001' -> '11111111111111111111111111110110' -> '11111111111111111111111111110111'
console.log('minus_nine: ' + minus_nine); // '-9'

// 32 significant bits representing the number are viewed as an *unsigned* 32-bit integer:
let x_: Number = new Number(-1 >>> 0); // Unsigned right shift that inserts '0' on the left (BTW: unsigned left shift does not exist!)
console.log('-1 >>> 0: ' + x_.toString(2)); // '11111111111111111111111111111111' -> (2^32 - 1) -> 4294967295

console.log(Number.MAX_SAFE_INTEGER.toString(2).length); // '53' since 'Number.MAX_SAFE_INTEGER' is equal to (2^53 - 1)
console.assert(Number.MAX_SAFE_INTEGER === -Number.MIN_SAFE_INTEGER);

console.log(Number.MAX_VALUE * 2 === Number.POSITIVE_INFINITY); // 'true'

console.log(isNaN(Math.sqrt(-1))); // 'true'
