"use strict";

function reassignment_example({given_name: x, surname: y}) {
    console.log(`x: ${x}, y: ${y}`);
}

reassignment_example({given_name: 'Franck', surname: 'Barbier'});

/** Functions */
var my_f = function (my_g) { // 'my_f' is defined
    console.log("'arguments' is a predefined array that lets the access to the passed arguments: " + arguments.length);
    my_g();
};
my_f(function () { // 'my_f' is called
    console.log("This no-name function plays the role of 'my_g' in 'my_f'");
});

const f = (g, i = 0) => { // 'f' is defined by means of a lambda expression
    console.log(g(i));
};
// Calling f...
f(function (my_i) { // 'f' is called with an anonymous function as parameter
    return ++my_i;
}/*, 1*/);
// Calling f, another style...
f(my_i => { // 'f' is called with an arrow as parameter
    return ++my_i;
}/*, 1*/);
// Calling f, yet another style...
f(my_i => ++my_i/*, 1*/); // Implicit 'return'

var my_function = function (my_parameter) {
    // Forbidden in strict mode (https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions/arguments/):
    // console.log(arguments.callee);
    return ++my_parameter;
};
console.assert(typeof my_function === 'function');
console.assert(my_function.constructor === Function);
console.log(my_function(0)); // '1' is displayed...

var her_function = function (her_parameter) {
    console.log("'this': " + this + " 'her_parameter': " + her_parameter);
    // Forbidden in strict mode (https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions/arguments/):
    // console.log(arguments.callee);
};
// First arg. plays the role of 'this' inside the function while '0' is substituted for 'her_parameter':
her_function.call("Edith", 0); // ''this': Edith 'her_parameter': 0' is displayed...
// First arg. plays the role of 'this' inside the function while '[0]' is a one-element array containing '0' as unique element:
her_function.apply("Edith", [0]); // ''this': Edith 'her_parameter': 0' is displayed...

const h = function (a = -1, ...bs /*, c*/ /* <- error! */) {
    let sum = a;
    bs.forEach(b => {
        console.log("b: " + b);
        sum += b;
    });
    console.log("Sum: " + sum);
};
// Note that default value for 'a' is useless:
h(1, 2, 3); // 'Sum: 6' is displayed, i.e., '1' is substituted for 'a'
h(...[1, 2, 3]); // <=> 'h(1, 2, 3);'
