const Dish = function (main_ingredient) {
    this.ingredients = [];
    this.main_ingredient = main_ingredient;
    this.add_ingredient = function (ingredient) {
        this.ingredients.push(ingredient);
    };
};
const lasagna = new Dish(Symbol("Pasta"));
console.log(lasagna.main_ingredient); // 'Symbol(Pasta)' is displayed
lasagna.add_ingredient(Symbol("Cheese"));
console.assert(lasagna.constructor === Dish);
console.assert(Object.getPrototypeOf(lasagna) === Dish.prototype); // "Prototype inheritance"...
console.log(Dish.prototype.constructor); // Constructor function
console.assert(Dish.prototype.constructor.name === "Dish");

class Dish_ { // From JavaScript 6...
    constructor(main_ingredient) {
        this.ingredients = [];
        this.main_ingredient = main_ingredient;
    }

    add_ingredient(ingredient) {
        this.ingredients.push(ingredient);
    };
}

const lasagna_ = new Dish_(Symbol("Pasta"));

let Franck = {};
Object.defineProperty(Franck, "surname", {value: "Barbier", enumerable: true, configurable: true, writable: true});
Object.defineProperty(Franck, "nickname", {value: "Bab", enumerable: true, configurable: false, writable: false});
try {
    Franck.nickname = "Other nickname";
} catch (error) { // JavaScript, bug:
    console.log(error.message); // '"nickname" is read-only'
}
try {
    delete Franck.nickname;
} catch (error) { // JavaScript, bug:
    console.log(error.message); // 'property "nickname" is non-configurable and can't be deleted'
}
Franck.skill = "JavaScript"; // Dynamic extension
console.log(JSON.stringify(Franck)); // '{"surname":"Barbier","nickname":"Bab","skill":"JavaScript"}'
delete Franck.skill; // Dynamic suppression
if ("skill" in Franck === false)
    console.log("'Franck.skill' no longer exists...");

const o = {};
Object.defineProperty(o, "simple_array", {
    value: [null, 0, "ABC"],
    enumerable: true,
    configurable: true,
    writable: true
});

o.simple_array_processing = function () {
    for (let i = 0; i < this.simple_array.length; i++)
        this.simple_array[i] = null; // Great, 'this' is set to the appropriate "current" object, i.e., 'o'!
};
o.simple_array_processing(); // Call...
console.log(JSON.stringify(o.simple_array)); // '[null,null,null]'

o.simple_array_processing = function () { // Erase prior version...
    this.simple_array.forEach(function (element) {
        console.assert(this !== o);
        console.log("What is 'this'? " + Object.getOwnPropertyNames(this).join(" - "));
    } /* ,this */); // Uncommenting leads to 'this === o'
};
o.simple_array_processing(); // Call...

o.simple_array_processing = function () { // Erase prior version...
    this.simple_array.forEach(element => { // 'this' in lambda expression is set *IN A STATIC WAY*
        console.assert(this === o);
        console.log("What is 'this'? " + Object.getOwnPropertyNames(this).join(" - "));
    } /* ,this */); // No need of injecting 'this' as second argument of 'forEach'
};
o.simple_array_processing(); // Call...

const my_iteration = function (element) {
    console.assert(this === o);
    console.log("What is 'this'? " + Object.getOwnPropertyNames(this).join(" - "));
};
o.simple_array_processing = function () { // Erase prior version...
    this.simple_array.forEach(my_iteration.bind(this));
};
o.simple_array_processing(); // Call...
